# Generated by Django 4.2.6 on 2024-01-31 08:30

import django_fsm
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("wbcrm", "0006_alter_activity_location"),
    ]

    operations = [
        migrations.AlterField(
            model_name="account",
            name="status",
            field=django_fsm.FSMField(
                choices=[("PENDING", "Pending"), ("OPEN", "Open"), ("CLOSE", "Close")],
                default="OPEN",
                max_length=50,
                verbose_name="Status",
            ),
        ),
    ]
